/*
  Q Light Controller
  artnetout.cpp
  
  Copyright (c) Heikki Junnila
                Simon Newton
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  Version 2 as published by the Free Software Foundation.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details. The license is
  in the file "COPYING".
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "artnetout.h"
#include "artnetthread.h"

#include "configureartnetout.h"
#include "../common/filehandler.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <qthread.h>
#include <qstring.h>
#include <qpoint.h>
#include <qpopupmenu.h>
#include <qfile.h>
#include <errno.h>
#include <assert.h>

#define CONF_FILE "artnetout.conf"

#define ID_CONFIGURE      10
#define ID_ACTIVATE       20

static QMutex _mutex;

//
// Exported functions
//
extern "C" OutputPlugin* create(t_plugin_id id)
{
  return new ArtNetOut(id);
}

extern "C" void destroy(OutputPlugin* object)
{
  delete object;
}

//
// Class implementation
//
ArtNetOut::ArtNetOut(t_plugin_id id) : OutputPlugin(id)
{
  m_thr = NULL ;
  m_name = QString("ArtNet Output");
  m_type = OutputType;
  m_version = 0x00010000;
  m_configDir = QString("~/.qlc/");

  memset(m_values, 0x00, KChannelMax) ;
  
}

ArtNetOut::~ArtNetOut()
{

}

/* start the thread */
int ArtNetOut::open()
{
  printf("open\n") ;
  // check this for error
  m_thr = new ArtNetThread(m_ip) ;
  m_thr->start() ;
  return 0;
}

/* terminate the thread */
int ArtNetOut::close()
{

  printf("close\n") ;
  if(m_thr == NULL)
	return -1 ;

  if(m_thr->running()) {
    m_thr->stop() ;
  }
  // wait for thread termination
  m_thr->wait() ;

  delete m_thr ;
  m_thr = NULL ;
  return 0;
}

/* check if thread is running */
bool ArtNetOut::isOpen()
{
  if(m_thr == NULL)
	return false ;

  return m_thr->running() ;
}

/* called on configure */
int ArtNetOut::configure()
{
  ConfigureArtNetOut* conf = new ConfigureArtNetOut(this);

  if (conf->exec() == QDialog::Accepted)
    {
      m_ip = conf->ip();			
      saveSettings();
    }

  delete conf;

  return 0;
}

/* return info string (allows html) */
QString ArtNetOut::infoText()
{
  QString t;
  QString str = QString::null;
  str += QString("<HTML><HEAD><TITLE>Plugin Info</TITLE></HEAD><BODY>");
  str += QString("<TABLE COLS=\"1\" WIDTH=\"100%\"><TR>");
  str += QString("<TD BGCOLOR=\"black\"><FONT COLOR=\"white\" SIZE=\"5\">");
  str += name() + QString("</FONT></TD></TR></TABLE>");
  str += QString("<TABLE COLS=\"2\" WIDTH=\"100%\">");
  str += QString("<TR>\n");
  str += QString("<TD><B>Version</B></TD>");
  str += QString("<TD>");
  t.setNum((version() >> 16) & 0xff);
  str += t + QString(".");
  t.setNum((version() >> 8) & 0xff);
  str += t + QString(".");
  t.setNum(version() & 0xff);
  str += t + QString("</TD>");
  str += QString("</TR>");

  str += QString("<TR>\n");
  str += QString("<TD><B>Status</B></TD>");
  str += QString("<TD>");
  if (isOpen() == true)
    {
      str += QString("<I>Active</I></TD>");
    }
  else
    {
      str += QString("Not Active</TD>");
    }
  str += QString("</TR>");

  str += QString("</TR>");
  str += QString("</TABLE>");
  str += QString("</BODY></HTML>");

  return str;
}

/* set config directory */
int ArtNetOut::setConfigDirectory(QString dir)
{
  m_configDir = dir;
  return 0;
}

/* save settings to config file */
int ArtNetOut::saveSettings()
{
  QString s;
  QString t;

  QString fileName = m_configDir + QString(CONF_FILE);
  qDebug(fileName);
  QFile file(fileName);

  if (file.open(IO_WriteOnly))
    {
      // Comment
      s = QString("# ArtNetOut Plugin Configuration\n");
      file.writeBlock((const char*) s, s.length());

      // Entry type
      s = QString("Entry = ") + name() + QString("\n");
      file.writeBlock((const char*) s, s.length());
	  
      s = QString("IP = ") + m_ip + QString("\n");
      file.writeBlock((const char*) s, s.length());

      file.close();
    }
  else
    {
      perror("file.open");
      qDebug("Unable to save ArtNetOut configuration");
    }

  return errno;
}


/* load settings from config file */
int ArtNetOut::loadSettings()
{
  QString fileName;
  QPtrList <QString> list;

  fileName = m_configDir + QString(CONF_FILE);

  FileHandler::readFileToList(fileName, list);

  for (QString* s = list.first(); s != NULL; s = list.next())
    {
      if (*s == QString("Entry"))
        {
          s = list.next();
          if (*s == name())
            {
              createContents(list);
            }
        }
    }

  return 0;
}

/* */
void ArtNetOut::createContents(QPtrList <QString> &list)
{
  for (QString* s = list.next(); s != NULL; s = list.next())
    {
      if (*s == QString("Entry"))
        {
          s = list.prev();
          break;
        } 
      else if (*s == QString("IP"))
       {
	     m_ip = *(list.next());
       }

    }
}

/* ok some sort of popup menu here, haven't found where it's used */
void ArtNetOut::contextMenu(QPoint pos)
{
  QPopupMenu* menu = new QPopupMenu();
  menu->insertItem("Configure...", ID_CONFIGURE);
  menu->insertSeparator();
  menu->insertItem("Activate", ID_ACTIVATE);

  connect(menu, SIGNAL(activated(int)), 
	  this, SLOT(slotContextMenuCallback(int)));
  menu->exec(pos, 0);
  delete menu;
}

/* handle menu click */
void ArtNetOut::slotContextMenuCallback(int item)
{
  switch(item)
    {
    case ID_CONFIGURE:
      configure();
      break;

    case ID_ACTIVATE:
      activate();
      break;

    default:
      break;
    }
}

/* start  */
void ArtNetOut::activate()
{
  emit activated(this);
}


/* Write a value to a channel
 *
 * takes the channel numbers and the value
 */
int ArtNetOut::writeChannel(t_channel channel, t_value value)
{
  int r ;
  _mutex.lock();

  m_values[channel] = value;

  if(m_thr == NULL)
	return 0 ;
  
  // write
  r = m_thr->write_dmx((uint8_t *) &m_values, CHANNELS ) ;
  
  _mutex.unlock();

  return r ;
}

/* Write num values starting from address
 *
 * takes the starting address, pointer to values and length
 */
int ArtNetOut::writeRange(t_channel address, t_value* values, t_channel num)
{
  int r ;
  assert(values);

  _mutex.lock();

  memcpy(m_values + address, values, num * sizeof(t_value));

  if(m_thr == NULL)
	return 0 ;
 
  // write to pipe
  r = m_thr->write_dmx((uint8_t *) &m_values, CHANNELS ) ;
  
  _mutex.unlock();

  return r;
}

/* Read a channel's value
 *
 *
 */
int ArtNetOut::readChannel(t_channel channel, t_value &value)
{
  _mutex.lock();
  value = m_values[channel];
  _mutex.unlock();

  return 0;
}

/* Read num channel's values starting from address
 *
 */
int ArtNetOut::readRange(t_channel address, t_value* values, t_channel num)
{
  assert(values);

  _mutex.lock();
  memcpy(values, m_values + address, num * sizeof(t_value));
  _mutex.unlock();

  return 0;
}
